@@ -36,20 +36,37 @@
 		values, \
 		C_ARRAY_SIZE(values) - isnullterminated, \
 		C_ARRAY_ITEMSIZE(values), \
-		match)
+		match, \
+		NULL, NULL, 0)
+
+#define DC_FILTER_INTERNAL_WITH_PARAMS(key, values, isnullterminated, match, params_dst, params_src) \
+	dc_filter_internal( \
+		key, \
+		values, \
+		C_ARRAY_SIZE(values) - isnullterminated, \
+		C_ARRAY_ITEMSIZE(values), \
+		match, \
+		params_dst, params_src, sizeof *(params_src))
 
 typedef int (*dc_match_t)(const void *, const void *);
 
-static int dc_filter_uwatec (dc_transport_t transport, const void *userdata);
-static int dc_filter_suunto (dc_transport_t transport, const void *userdata);
-static int dc_filter_shearwater (dc_transport_t transport, const void *userdata);
-static int dc_filter_hw (dc_transport_t transport, const void *userdata);
-static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata);
-static int dc_filter_garmin (dc_transport_t transport, const void *userdata);
-static int dc_filter_mares (dc_transport_t transport, const void *userdata);
-static int dc_filter_divesystem (dc_transport_t transport, const void *userdata);
-static int dc_filter_oceanic (dc_transport_t transport, const void *userdata);
-static int dc_filter_deepblu (dc_transport_t transport, const void *userdata);
+typedef int (*dc_filter_t) (dc_transport_t transport, const void *userdata, void *params);
+
+static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_suunto (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_shearwater (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_hw (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_mares (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_divesystem (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_mclean (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_atomic (dc_transport_t transport, const void *userdata, void *params);
+
+// Not merged upstream yet
+static int dc_filter_garmin (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_deepblu (dc_transport_t transport, const void *userdata, void *params);
+static int dc_filter_oceans(dc_transport_t transport, const void *userdata, void *params);
 
 static dc_status_t dc_descriptor_iterator_next (dc_iterator_t *iterator, void *item);
 
@@ -249,6 +266,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Oceanic",  "Veo 4.0",             DC_FAMILY_OCEANIC_ATOM2, 0x4654, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
 	{"Sherwood", "Wisdom 4",            DC_FAMILY_OCEANIC_ATOM2, 0x4655, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
 	{"Oceanic",  "Pro Plus 4",          DC_FAMILY_OCEANIC_ATOM2, 0x4656, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
+	{"Aqualung", "i470TC",              DC_FAMILY_OCEANIC_ATOM2, 0x4743, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLE, dc_filter_oceanic},
 	/* Mares Nemo */
 	{"Mares", "Nemo",         DC_FAMILY_MARES_NEMO, 0, DC_TRANSPORT_SERIAL, NULL},
 	{"Mares", "Nemo Steel",   DC_FAMILY_MARES_NEMO, 0, DC_TRANSPORT_SERIAL, NULL},
@@ -314,8 +332,8 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Dive Rite", "NiTek Trio", DC_FAMILY_ZEAGLE_N2ITION3, 0, DC_TRANSPORT_SERIAL, NULL},
 	{"Scubapro",  "XTender 5",  DC_FAMILY_ZEAGLE_N2ITION3, 0, DC_TRANSPORT_SERIAL, NULL},
 	/* Atomic Aquatics Cobalt */
-	{"Atomic Aquatics", "Cobalt", DC_FAMILY_ATOMICS_COBALT, 0, DC_TRANSPORT_USB, NULL},
-	{"Atomic Aquatics", "Cobalt 2", DC_FAMILY_ATOMICS_COBALT, 2, DC_TRANSPORT_USB, NULL},
+	{"Atomic Aquatics", "Cobalt",   DC_FAMILY_ATOMICS_COBALT, 0, DC_TRANSPORT_USB, dc_filter_atomic},
+	{"Atomic Aquatics", "Cobalt 2", DC_FAMILY_ATOMICS_COBALT, 2, DC_TRANSPORT_USB, dc_filter_atomic},
 	/* Shearwater Predator */
 	{"Shearwater", "Predator", DC_FAMILY_SHEARWATER_PREDATOR, 2, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH, dc_filter_shearwater},
 	/* Shearwater Petrel */
@@ -326,6 +344,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Shearwater", "Perdix AI", DC_FAMILY_SHEARWATER_PETREL, 6, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	{"Shearwater", "Nerd 2",    DC_FAMILY_SHEARWATER_PETREL, 7, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	{"Shearwater", "Teric",     DC_FAMILY_SHEARWATER_PETREL, 8, DC_TRANSPORT_BLE, dc_filter_shearwater},
+	{"Shearwater", "Peregrine", DC_FAMILY_SHEARWATER_PETREL, 9, DC_TRANSPORT_BLE, dc_filter_shearwater},
 	/* Dive Rite NiTek Q */
 	{"Dive Rite", "NiTek Q",   DC_FAMILY_DIVERITE_NITEKQ, 0, DC_TRANSPORT_SERIAL, NULL},
 	/* Citizen Hyper Aqualand */
@@ -378,10 +397,21 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Cochran", "EMC-20H",      DC_FAMILY_COCHRAN_COMMANDER, 5, DC_TRANSPORT_SERIAL, NULL},
 	/* Tecdiving DiveComputer.eu */
 	{"Tecdiving", "DiveComputer.eu", DC_FAMILY_TECDIVING_DIVECOMPUTEREU, 0, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH, dc_filter_tecdiving},
+	/* McLean Extreme */
+	{ "McLean", "Extreme", DC_FAMILY_MCLEAN_EXTREME, 0, DC_TRANSPORT_SERIAL | DC_TRANSPORT_BLUETOOTH | DC_TRANSPORT_BLE, dc_filter_mclean},
+	/* Liquivision */
+	{"Liquivision", "Xen",  DC_FAMILY_LIQUIVISION_LYNX, 0, DC_TRANSPORT_SERIAL, NULL},
+	{"Liquivision", "Xeo",  DC_FAMILY_LIQUIVISION_LYNX, 1, DC_TRANSPORT_SERIAL, NULL},
+	{"Liquivision", "Lynx", DC_FAMILY_LIQUIVISION_LYNX, 2, DC_TRANSPORT_SERIAL, NULL},
+	{"Liquivision", "Kaon", DC_FAMILY_LIQUIVISION_LYNX, 3, DC_TRANSPORT_SERIAL, NULL},
+
+	// Not merged upstream yet
 	/* Garmin */
 	{"Garmin", "Descent Mk1", DC_FAMILY_GARMIN, 2859, DC_TRANSPORT_USBSTORAGE, dc_filter_garmin},
 	/* Deepblu */
 	{"Deepblu", "Cosmiq+", DC_FAMILY_DEEPBLU, 0, DC_TRANSPORT_BLE, dc_filter_deepblu},
+	/* Oceans S1 */
+	{ "Oceans", "S1", DC_FAMILY_OCEANS_S1, 0, DC_TRANSPORT_BLE, dc_filter_oceans },
 };
 
 static int
@@ -458,13 +488,16 @@ dc_match_oceanic (const void *key, const void *value)
 }
 
 static int
-dc_filter_internal (const void *key, const void *values, size_t count, size_t size, dc_match_t match)
+dc_filter_internal (const void *key, const void *values, size_t count, size_t size, dc_match_t match, void *params_dst, const void *params_src, size_t params_size)
 {
 	if (key == NULL)
 		return 0;
 
 	for (size_t i = 0; i < count; ++i) {
 		if (match (key, (const unsigned char *) values + i * size)) {
+			if (params_src && params_dst) {
+				memcpy (params_dst, params_src, params_size);
+			}
 			return 1;
 		}
 	}
@@ -479,7 +512,7 @@ static const char * const rfcomm[] = {
 	NULL
 };
 
-static int dc_filter_uwatec (dc_transport_t transport, const void *userdata)
+static int dc_filter_uwatec (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const irda[] = {
 		"Aladin Smart Com",
@@ -514,7 +547,7 @@ static int dc_filter_uwatec (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_suunto (dc_transport_t transport, const void *userdata)
+static int dc_filter_suunto (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const dc_usb_desc_t usbhid[] = {
 		{0x1493, 0x0030}, // Eon Steel
@@ -536,7 +569,7 @@ static int dc_filter_suunto (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_hw (dc_transport_t transport, const void *userdata)
+static int dc_filter_hw (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"OSTC",
@@ -552,7 +585,7 @@ static int dc_filter_hw (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_shearwater (dc_transport_t transport, const void *userdata)
+static int dc_filter_shearwater (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"Predator",
@@ -561,6 +594,7 @@ static int dc_filter_shearwater (dc_transport_t transport, const void *userdata)
 		"NERD 2",
 		"Perdix",
 		"Teric",
+		"Peregrine",
 	};
 
 	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
@@ -572,7 +606,7 @@ static int dc_filter_shearwater (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata)
+static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"DiveComputer",
@@ -587,20 +621,7 @@ static int dc_filter_tecdiving (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_garmin (dc_transport_t transport, const void *userdata)
-{
-	static const dc_usb_desc_t usbhid[] = {
-		{0x091e, 0x2b2b}, // Garmin Descent Mk1
-	};
-
-	if (transport == DC_TRANSPORT_USBSTORAGE) {
-		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usb);
-	}
-
-	return 1;
-}
-
-static int dc_filter_mares (dc_transport_t transport, const void *userdata)
+static int dc_filter_mares (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"Mares bluelink pro",
@@ -614,7 +635,7 @@ static int dc_filter_mares (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_divesystem (dc_transport_t transport, const void *userdata)
+static int dc_filter_divesystem (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"DS",
@@ -627,7 +648,7 @@ static int dc_filter_divesystem (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_oceanic (dc_transport_t transport, const void *userdata)
+static int dc_filter_oceanic (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const unsigned int model[] = {
 		0x4552, // Oceanic Pro Plus X
@@ -639,6 +660,7 @@ static int dc_filter_oceanic (dc_transport_t transport, const void *userdata)
 		0x4654, // Oceanic Veo 4.0
 		0x4655, // Sherwood Wisdom 4
 		0x4656, // Oceanic Pro Plus 4
+		0x4743, // Aqualung i470TC
 	};
 
 	if (transport == DC_TRANSPORT_BLE) {
@@ -648,7 +670,53 @@ static int dc_filter_oceanic (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
-static int dc_filter_deepblu (dc_transport_t transport, const void *userdata)
+static int dc_filter_mclean(dc_transport_t transport, const void *userdata, void *params)
+{
+	static const char * const bluetooth[] = {
+		"McLean Extreme",
+	};
+
+	if (transport == DC_TRANSPORT_BLUETOOTH || transport == DC_TRANSPORT_BLE) {
+		return DC_FILTER_INTERNAL (userdata, bluetooth, 0, dc_match_name);
+	} else if (transport == DC_TRANSPORT_SERIAL) {
+		return DC_FILTER_INTERNAL(userdata, rfcomm, 1, dc_match_devname);
+	}
+
+	return 1;
+}
+
+static int dc_filter_atomic (dc_transport_t transport, const void *userdata, void *params)
+{
+	static const dc_usb_desc_t usb[] = {
+		{0x0471, 0x0888}, // Atomic Aquatics Cobalt
+	};
+
+	static const dc_usb_params_t usb_params = {
+		0, 0x82, 0x02
+	};
+
+	if (transport == DC_TRANSPORT_USB) {
+		return DC_FILTER_INTERNAL_WITH_PARAMS (userdata, usb, 0, dc_match_usb, params, &usb_params);
+	}
+
+	return 1;
+}
+
+// Not merged upstream yet
+static int dc_filter_garmin (dc_transport_t transport, const void *userdata, void *params)
+{
+	static const dc_usb_desc_t usbhid[] = {
+		{0x091e, 0x2b2b}, // Garmin Descent Mk1
+	};
+
+	if (transport == DC_TRANSPORT_USBSTORAGE) {
+		return DC_FILTER_INTERNAL (userdata, usbhid, 0, dc_match_usb);
+	}
+
+	return 1;
+}
+
+static int dc_filter_deepblu (dc_transport_t transport, const void *userdata, void *params)
 {
 	static const char * const bluetooth[] = {
 		"COSMIQ",
@@ -661,6 +729,19 @@ static int dc_filter_deepblu (dc_transport_t transport, const void *userdata)
 	return 1;
 }
 
+static int dc_filter_oceans(dc_transport_t transport, const void* userdata, void *params)
+{
+	static const char* const ble[] = {
+		"S1",
+	};
+
+	if (transport == DC_TRANSPORT_BLE) {
+		return DC_FILTER_INTERNAL(userdata, ble, 0, dc_match_prefix);
+	}
+
+	return 1;
+}
+
 dc_status_t
 dc_descriptor_iterator (dc_iterator_t **out)
 {
@@ -752,11 +833,11 @@ dc_descriptor_get_transports (dc_descriptor_t *descriptor)
 	return descriptor->transports;
 }
 
-dc_filter_t
-dc_descriptor_get_filter (dc_descriptor_t *descriptor)
+int
+dc_descriptor_filter (dc_descriptor_t *descriptor, dc_transport_t transport, const void *userdata, void *params)
 {
-	if (descriptor == NULL)
-		return NULL;
+	if (descriptor == NULL || descriptor->filter == NULL)
+		return 1;
 
-	return descriptor->filter;
+	return descriptor->filter (transport, userdata, params);
 }
