@@ -55,6 +55,8 @@
 #define LOG_RECORD_INFO_EVENT      0x30
 #define LOG_RECORD_FINAL           0xFF
 
+#define INFO_EVENT_TAG_LOG         38
+
 #define SZ_BLOCK   0x80
 #define SZ_SAMPLE_PREDATOR  0x10
 #define SZ_SAMPLE_PETREL    0x20
@@ -77,8 +79,6 @@
 #define PREDATOR 2
 #define PETREL   3
 
-#define INFO_EVENT_TAG_LOG 38
-
 #define UNDEFINED 0xFFFFFFFF
 
 typedef struct shearwater_predator_parser_t shearwater_predator_parser_t;
@@ -562,7 +562,10 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 	}
 
 	// Verify the required opening/closing records.
-	for (unsigned int i = 0; i < NRECORDS - 2; ++i) {
+	// At least in firmware v71 and newer, Petrel and Petrel 2 also use PNF,
+	// and there opening/closing record 5 (which contains AI information plus
+	// the sample interval) don't appear to exist - so don't mark them as required
+	for (unsigned int i = 0; i <= 4; ++i) {
 		if (parser->opening[i] == UNDEFINED || parser->closing[i] == UNDEFINED) {
 			ERROR (abstract->context, "Opening or closing record %u not found.", i);
 			return DC_STATUS_DATAFORMAT;
@@ -739,7 +742,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 	// Sample interval.
 	unsigned int time = 0;
 	unsigned int interval = 10;
-	if (parser->pnf && parser->logversion >= 9) {
+	if (parser->pnf && parser->logversion >= 9 && parser->opening[5] != UNDEFINED) {
 		interval = array_uint16_be (data + parser->opening[5] + 23);
 		if (interval % 1000 != 0) {
 			ERROR (abstract->context, "Unsupported sample interval (%u ms).", interval);
@@ -754,41 +757,6 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 	while (offset + parser->samplesize <= length) {
 		dc_sample_value_t sample = {0};
 
-		// stop parsing if we see the end block
-		if (pnf && data[offset] == LOG_RECORD_FINAL && data[offset + 1] == 0xFD)
-			break;
-
-		if (pnf && data[offset] == LOG_RECORD_INFO_EVENT) {
-			// additional events defined in PNF
-			INFO(abstract->context, "PNF INFO_EVENT ID %u time %u W1 %u W2 %u", data[offset + 1],
-											    array_uint32_be (data + offset + 4),
-											    array_uint32_be (data + offset + 8),
-											    array_uint32_be (data + offset + 12));
-			if (data[offset + 1] == INFO_EVENT_TAG_LOG) {
-				// this is a TAG
-				// its time is a unix timestamp, so we need to subtract the dive start time
-				unsigned int tag_time = array_uint32_be (data + offset + 4) - array_uint32_be (data + parser->opening[0] + 12);
-				unsigned int tag_heading = array_uint32_be (data + offset + 8);
-				unsigned int tag_type = array_uint32_be (data + offset + 12);
-				// heading is only valid if 0..360
-				// type is only valid if 0..5
-				if (tag_heading <= 360 && tag_type <= 5) {
-					// encode this as a bookmark event, using the flags to capture the type and value for heading
-					sample.event.type = SAMPLE_EVENT_BOOKMARK;
-					sample.event.time = tag_time;
-					sample.event.flags = (tag_type + 1) << SAMPLE_FLAGS_TYPE_SHIFT; // 0 means it isn't a tag
-					sample.event.value = tag_heading;
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
-				}
-			}
-			offset += parser->samplesize;
-			continue;
-		}
-		// Ignore blocks that aren't dive samples
-		if (pnf && data[offset] != LOG_RECORD_DIVE_SAMPLE) {
-			offset += parser->samplesize;
-			continue;
-		}
 		// Ignore empty samples.
 		if (array_isequal (data + offset, parser->samplesize, 0x00)) {
 			offset += parser->samplesize;
@@ -798,6 +766,10 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 		// Get the record type.
 		unsigned int type = pnf ? data[offset] : LOG_RECORD_DIVE_SAMPLE;
 
+		// stop parsing if we see the end block
+		if (type == LOG_RECORD_FINAL && data[offset + 1] == 0xFD)
+			break;
+
 		if (type == LOG_RECORD_DIVE_SAMPLE) {
 			// Time (seconds).
 			time += interval;
@@ -961,9 +933,30 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 				sample.temperature = temperature / 10.0;
 				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 			}
+		} else if (type == LOG_RECORD_INFO_EVENT) {
+			unsigned int event = data[offset + 1];
+			unsigned int timestamp = array_uint32_be (data + offset + 4);
+			unsigned int w1 = array_uint32_be (data + offset + 8);
+			unsigned int w2 = array_uint32_be (data + offset + 12);
+
+			if (event == INFO_EVENT_TAG_LOG) {
+				// Compass heading
+				if (w1 != 0xFFFFFFFF) {
+					sample.bearing = w1;
+					if (callback) callback (DC_SAMPLE_BEARING, sample, userdata);
+				}
+
+				// Tag
+				sample.event.type = SAMPLE_EVENT_BOOKMARK;
+				sample.event.time = 0;
+				sample.event.flags = 0;
+				sample.event.value = w2;
+				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+			}
 		}
 
 		offset += parser->samplesize;
 	}
+
 	return DC_STATUS_SUCCESS;
 }
